{           1998, 1999 Kwik-Rite Development.  All rights reserved.           }

unit ConsoleUtils;

interface

uses
  Windows;

type
  TConsoleColour = (ccBlack, ccRed, ccGreen, ccBlue, ccMagenta,
    ccYellow, ccCyan, ccWhite);

  TConsoleInfo = record
    Width : Integer;
    Height : Integer;
    CursorLeft : Integer;
    CursorTop : Integer;
    Foreground : TConsoleColour;
    Background : TConsoleColour;
  end;

procedure cuClearScreen;
procedure cuSetTitle(Title : string);
procedure cuWriteLnCentred(Text : string);
procedure cuSetTextColour(Foreground, Background : TConsoleColour);
procedure cuSetOutputColour(Foreground, Background : TConsoleColour);
procedure cuSetCursorPosition(Left, Top : Integer);
procedure cuHideCursor;
procedure cuShowCursor;
function cuGetConsoleInfo : TConsoleInfo;
procedure cuReadLnPassword(var Password:String; PasswordChar : string);

implementation

function ColoursToAttributes(Foreground, Background : TConsoleColour) : Word;
var
  wAttributes : Word;
begin
  wAttributes := 0;
  case Foreground of
    ccRed : wAttributes := wAttributes or FOREGROUND_RED;
    ccGreen : wAttributes := wAttributes or FOREGROUND_GREEN;
    ccBlue : wAttributes := wAttributes or FOREGROUND_BLUE;
    ccMagenta : wAttributes := wAttributes or FOREGROUND_RED or FOREGROUND_BLUE;
    ccYellow : wAttributes := wAttributes or FOREGROUND_RED or FOREGROUND_GREEN;
    ccCyan : wAttributes := wAttributes or FOREGROUND_GREEN or FOREGROUND_BLUE;
    ccWhite :
      wAttributes := wAttributes or FOREGROUND_RED or FOREGROUND_BLUE or
        FOREGROUND_GREEN;
  end;
  case Background of
    ccRed : wAttributes := wAttributes or BACKGROUND_RED;
    ccGreen : wAttributes := wAttributes or BACKGROUND_GREEN;
    ccBlue : wAttributes := wAttributes or BACKGROUND_BLUE;
    ccMagenta : wAttributes := wAttributes or BACKGROUND_RED or BACKGROUND_BLUE;
    ccYellow : wAttributes := wAttributes or BACKGROUND_RED or BACKGROUND_GREEN;
    ccCyan : wAttributes := wAttributes or BACKGROUND_GREEN or BACKGROUND_BLUE;
    ccWhite :
      wAttributes := wAttributes or BACKGROUND_RED or BACKGROUND_BLUE or
        BACKGROUND_GREEN;
  end;
  Result := wAttributes;
end;

procedure AttributesToColours(Attributes : Word; var Foreground,
  Background : TConsoleColour);
begin
  Foreground := ccBlack;
  if (Attributes and (FOREGROUND_GREEN or FOREGROUND_RED or
    FOREGROUND_BLUE)) <> 0 then
    Foreground := ccWhite
  else if (Attributes and (FOREGROUND_RED or FOREGROUND_BLUE)) <> 0 then
    Foreground := ccMagenta
  else if (Attributes and (FOREGROUND_RED or FOREGROUND_GREEN)) <> 0 then
    Foreground := ccYellow
  else if (Attributes and (FOREGROUND_BLUE or FOREGROUND_GREEN)) <> 0 then
    Foreground := ccCyan
  else if (Attributes and FOREGROUND_RED) <> 0 then
    Foreground := ccRed
  else if (Attributes and FOREGROUND_GREEN) <> 0 then
    Foreground := ccGreen
  else if (Attributes and FOREGROUND_BLUE) <> 0 then
    Foreground := ccBlue;

  Background := ccBlack;
  if (Attributes and (BACKGROUND_GREEN or BACKGROUND_RED or
    BACKGROUND_BLUE)) <> 0 then
    Background := ccWhite
  else if (Attributes and (BACKGROUND_RED or BACKGROUND_BLUE)) <> 0 then
    Background := ccMagenta
  else if (Attributes and (BACKGROUND_RED or BACKGROUND_GREEN)) <> 0 then
    Background := ccYellow
  else if (Attributes and (BACKGROUND_BLUE or BACKGROUND_GREEN)) <> 0 then
    Background := ccCyan
  else if (Attributes and BACKGROUND_BLUE) <> 0 then
    Background := ccBlue
  else if (Attributes and BACKGROUND_GREEN) <> 0 then
    Background := ccGreen
  else if (Attributes and BACKGROUND_RED) <> 0 then
    Background := ccRed;
end;

function cuGetConsoleInfo : TConsoleInfo;
var
  ConsInfo : TConsoleScreenBufferInfo;
begin
  GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), ConsInfo);
  Result.Width := ConsInfo.dwSize.X;
  Result.Height := ConsInfo.dwSize.Y;
  Result.CursorLeft := ConsInfo.dwCursorPosition.X;
  Result.CursorTop := ConsInfo.dwCursorPosition.Y;
  AttributesToColours(ConsInfo.wAttributes, Result.Foreground, Result.Background);
end;

procedure cuClearScreen;
var
  sbi : TConsoleScreenBufferInfo;
  i : Integer;
begin
  GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), sbi);
  for i := 0 to sbi.dwSize.y do
    writeln(StringOfChar(' ', sbi.dwSize.x));
end;

procedure cuSetTitle(Title : string);
begin
  SetConsoleTitle(PChar(Title));
end;

procedure cuSetTextColour(Foreground, Background : TConsoleColour);
begin
  SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE),
    ColoursToAttributes(Foreground, Background));
end;

procedure cuSetOutputColour(Foreground, Background : TConsoleColour);
var
  Attributes : WORD;
  StartPos : COORD;
  dwNoWritten : DWORD;
  ConsoleInfo : TConsoleInfo;
begin
  Attributes := ColoursToAttributes(Foreground, Background);
  StartPos.X := 0;
  StartPos.Y := 0;
  ConsoleInfo := cuGetConsoleInfo;
  FillConsoleOutputAttribute(GetStdHandle(STD_OUTPUT_HANDLE), Attributes,
    (ConsoleInfo.Width * ConsoleInfo.Height), StartPos, dwNoWritten);
end;

procedure cuSetCursorPosition(Left, Top : Integer);
var
  NewPos : COORD;
begin
  NewPos.X := Left;
  NewPos.Y := Top;
  SetConsoleCursorPosition(GetStdHandle(STD_OUTPUT_HANDLE), NewPos);
end;

procedure cuHideCursor;
var
  CursorInfo : TConsoleCursorInfo;
begin
  GetConsoleCursorInfo(GetStdHandle(STD_OUTPUT_HANDLE), CursorInfo);
  CursorInfo.bVisible := false;
  SetConsoleCursorInfo(GetStdHandle(STD_OUTPUT_HANDLE), CursorInfo);
end;

procedure cuShowCursor;
var
  CursorInfo : TConsoleCursorInfo;
begin
  GetConsoleCursorInfo(GetStdHandle(STD_OUTPUT_HANDLE), CursorInfo);
  CursorInfo.bVisible := true;
  SetConsoleCursorInfo(GetStdHandle(STD_OUTPUT_HANDLE), CursorInfo);
end;

procedure cuWriteLnCentred(Text : string);
var
  InsertText : string;
  InsertNum : Integer;
  n : Integer;
  sbi : TConsoleScreenBufferInfo;
begin
  GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), sbi);
  InsertNum := (sbi.dwSize.X - Length(Text)) div 2;
  InsertText := '';
  for n := 1 to InsertNum do
    InsertText := InsertText + ' ';
  InsertText := InsertText + Text;
  WriteLn(InsertText);
end;

procedure cuReadLnPassword(var Password:String; PasswordChar : string);
var
  ev : TInputrecord;
  events_read : DWORD;
  done : Boolean;
  ConsInfo:TConsoleInfo;
  Showing:Boolean;
  n:Integer;
begin
  done := False;
  Showing:=False;
  Password := '';
  ConsInfo:=cuGetConsoleInfo;

  repeat
    ReadConsoleInput(GetStdhandle(STD_INPUT_HANDLE),
      ev, 1, events_read);
    if ev.Eventtype = key_Event then
      with ev.Event.KeyEvent do
        if bKeyDown then
          begin
            done := wVirtualKeyCode = VK_RETURN;
            if wVirtualKeyCode = VK_F5 then
              begin
                Showing:=not Showing;
                if Showing then
                  begin
                    cuSetCursorPosition(ConsInfo.CursorLeft, ConsInfo.CursorTop);
                    Write(Password);
                  end
                else
                  begin
                    cuSetCursorPosition(ConsInfo.CursorLeft, ConsInfo.CursorTop);
                    for n:=1 to Length(Password) do
                      Write(PasswordChar);
                  end;
              end;
            if wVirtualKeyCode = VK_BACK then
              begin
                Delete(Password, Length(Password), 1);
                if Showing then
                  begin
                    cuSetCursorPosition(ConsInfo.CursorLeft, ConsInfo.CursorTop);
                    Write(Password);
                    Write(' ');
                    cuSetCursorPosition(ConsInfo.CursorLeft, ConsInfo.CursorTop);
                    Write(Password);
                  end
                else
                  begin
                    cuSetCursorPosition(ConsInfo.CursorLeft, ConsInfo.CursorTop);
                    for n:=1 to Length(Password) do
                      Write(PasswordChar);
                    Write(' ');
                    cuSetCursorPosition(ConsInfo.CursorLeft+Length(Password), ConsInfo.CursorTop);
                  end;
              end
            else if not done and (AsciiChar <> #0) then
              begin
                if Showing then
                  Write(AsciiChar)
                else
                  Write(PasswordChar);
                Password := Password + AsciiChar;
              end;
          end;
  until done;
end;

end.

