unit krSdStructures;

interface

uses
  Classes, SysUtils, Windows;

const
  CipherType : array[0..8] of string = ('Summer v1', 'Blowfish 256 bit', 'TEA (16 round) 128 bit',
    'TEA (32 round) 128 bit', 'IDEA - AscomTech AG 128 bit', 'DES 56 bit', 'Square 128 bit',
    'Mitsubishi Misty1 128 bit', 'Triple DES 168 bit');

type
  EScramDiskError = Exception;
  ESDWrongVersion = EScramDiskError;
  ESDVxdNotFound = EScramDiskError;
  ESDVxdNotConnected = EScramDiskError;
  ESDDriveInUse = EScramDiskError;

  TDeviceInfo = packed record
    dr_lett1 : char;
    dr_lett1status : char;
    dr_lett2 : char;
    dr_lett2status : char;
    dr_lett3 : char;
    dr_lett3status : char;
    dr_lett4 : char;
    dr_lett4status : char;
    dr_lett5 : char;
    dr_lett5status : char;
    dr_lett6 : char;
    dr_lett6status : char;
    dr_lett7 : char;
    dr_lett7status : char;
    dr_lett8 : char;
    dr_lett8status : char;
  end;

  TMountType = (mtNotMounted, mtPartition, mtSVLFile, mtWAVFile);

  TSlotInfo = packed record
    SlotNo : Integer;
    MountType : TMountType;
    HostDriveLetter : string;
    DriveMountedAs : string;
    VolumeName : string;
    CipherType : string;
    FileName : string;
    Comments : string;
    PreferredDrive : string;
    NoAccesses : Integer;
    LastAccess : TDateTime;
  end;

  TDeviceIOControlRegisters = packed record
    EBX : DWORD;
    EDX : DWORD;
    ECX : DWORD;
    EAX : DWORD;
    EDI : DWORD;
    ESI : DWORD;
    Flags : DWORD;
  end;

  TParamBlock = packed record
    Operation : Integer;
    NumLocks : Integer;
  end;

  TMountFileStruct = packed record
    DCB : Integer;
    Reserved1 : Integer; //reserved....
    Reserved2 : Integer; //reserved....
    WavOffset : Integer; // this has the offset to the WAV data if WAV (|0x80000000 if 4 bit wav)
    // Win32 needs tocalculate wavoffset. See Scramdisk source code...
    FileName : PChar; // string of file to mount.
    MountDrive : Integer;
    MountSlot : Integer;
    MountResult : Integer; // set to 3 if file mounts ok Bit 8 set if WAV and buffer cannot allocate...
    // Set to 0x83 (bit 7 set) if file ALREADY mounted.....
    // Set to 0 if no slots free
    // 1 Not mounted
    Critical : PChar; // Points at 2k critical data, for SKF mount or nil for normal
  end;

  TInfoStruct = packed record
    SlotNo : Integer;
    infotext : array[0..255] of char; // disk owner
    accessed : Integer; // any user...   4
    preferredletter : Integer; // any user	  4
    magiclow : Integer; // any user	  4
    magichigh : Integer; // any user	  4
    newtime : TSystemTime; //any user    16 bytes
    TimeValid1 : Integer; // any user	  4 bytes
    oldtime : TSystemTime; //any user	  16 bytes
    TimeValid2 : Integer; //any user	  4
    newmode : Integer; //any user	  4
    sumLOW : Cardinal;
    sumHIGH : Cardinal;
    sum3 : Cardinal; //dummy
    sum4 : Cardinal; //dummy
    invalidsum : Char;
    owner : Char;
    spare2 : Char;
    spare3 : Char;
    Overrun : array[0..200] of char; // Overrun buffer
  end;
  PInfoStruct = ^TInfoStruct;

  TWavInfo = packed record
    DataPointer : PChar;
    FileOffset : Cardinal;
    NumChannels : Word;
    WavType : Word;
    BitsPerSample : Word;
    Length : Byte;
  end;

  TDDB = packed record
    DDB_phys_addr : Cardinal;
    DDB_Next_DDB : Cardinal;
    DDB_Next_DDB_init : Cardinal;
    DDB_dcb_ptr : Cardinal;
    DDB_number_buses : Byte;
    DDB_ios_flags : Byte;
    DDB_sig : Word;
    DDB_dvt : Pointer;
    DDB_devnode_ptr : Pointer;
    DDB_reserved : Pointer;
  end;

  TSlotInfoEx = packed record
    DriveMountedFrom : string;
    DriveMountedAs : string;
    VolumeName : string;
  end;

  TCryptVol = packed record
    cipher : Integer;
    index : Integer;
    wavoffset : Integer;
    ldcb : Pointer;
    linkediop : Integer;
    booted : Integer;
    driveinuse : Integer;
    drive : Integer;

    cryptsectorfirst : Cardinal;
    cryptsectorlast : Cardinal;
    physdevDCB : Pointer;
    mainkeys : array[0..2047] of Char;
    mountfilehandle : Integer;
    devicetype : Integer;

    SKFCreateTime : Integer;
    digestindex : Integer;
    spare2 : Integer;
    spare3 : Integer;

    volname : array[0..15] of Char;
    digest : array[0..19] of Char;
    spare : array[0..139] of Char;

    addb : TDDB;

    logicaldcb : array[0..298] of byte;
    mounted_file_name : array[0..511] of Char;
  end;

const
  VWIN32_DIOC_DOS_IOCTL = 1;
  PD_GETCVS = 1;
  PD_INQUIRE_DEVICES = 2;
  NUMBER_MOUNT_TYPES = 3;
  REQMON_INQUIRE_DEVICES = 4;
  INQUIRE_CRYPT_DEVICES = 4;
  SUPPLY_PASSWORD = 5;
  CLOSE_CRYPTED_DEVICE1 = 6;
  CLOSE_CRYPTED_DEVICE2 = 7;
  CLOSE_CRYPTED_DEVICE3 = 8;
  CLOSE_CRYPTED_DEVICE4 = 9;
  PUTCVS = 10;
  DISKIO = 11;
  GETDCBINFO = 12;
  FINDDRIVELETTER = 13;
  DISMOUNTDRIVE = 14;
  MOUNTFILE = 15;
  GETSHIFTCONST = 16;
  CLEARPASSWORDS = 17;
  MOUNTNEWDISK = 18;
  BORDER = 19;
  VALIDATEPASSWORD = 20;
  APPQUERY = 21;
  VOLQUERY = 22;
  GETDRIVERVERSION = 23;
  SETTIMEOUT = 24;
  RELEASETIMESLICE = 25;
  READINFOBLOCK = 26;
  WRITEINFOBLOCK = 27;
  R0FILEIO = 28;
  RING0CALLBACK = 29;
  UNINSTALLSCRAMDISK = 30;
  SETKEYFILEPASSWORD = 31;
  GETKEYFILEPASSWORD = 32;
  GETPASSWORDBUFFER = 33;
  GETINKEY = 34;
  SUPPLYDATEANDTIME = 35;
  SUPPLYCLICKEDFILENAME = 36;
  SENDFORCEQUIT = 37;
  REVOKESKFDATA = 38;
  RAWDEVICEDATA = 39;
  CHANGEPASSWORDS = 40;
  CLOSE_CRYPTED_DEVICE_N = 41;
  APP_REQUEST_REFRESH = 42;
  INQUIRE_SLOT = 43;
  E_NOT_CONNECTED = 'ScramDisk not connected - set Active to TRUE first';
  E_DRIVE_IN_USE = 'Close all windows and applications used by this drive first';

implementation

end.

